@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-help.c,v 1.7 2006/05/29 10:14:28 paolo Exp $
+ * $Id: gedit-commands-help.c,v 1.8 2006/06/18 08:57:06 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -57,6 +57,8 @@ _gedit_cmd_help_about (GtkAction   *action,
 	static const gchar * const authors[] = {
 		"Paolo Maggi <paolo@gnome.org>",
 		"Paolo Borelli <pborelli@katamail.com>",
+		"Steve Fr\303\251cinaux  <steve@istique.net>",
+		"Jesse van den Kieboom  <jesse@icecrew.nl>",
 		"James Willcox <jwillcox@gnome.org>",
 		"Chema Celorio",
 		"Federico Mena Quintero <federico@novell.com>",
