@@ -2,7 +2,7 @@
  * gedit-document-saver.c
  * This file is part of gedit
  *
- * Copyright (C) 2005 - Paolo Borelli and Paolo Maggi
+ * Copyright (C) 2005-2006 - Paolo Borelli and Paolo Maggi
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +21,11 @@
  */
 
 /*
- * Modified by the gedit Team, 2005. See the AUTHORS file for a 
+ * Modified by the gedit Team, 2005-2006. See the AUTHORS file for a 
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document-saver.c,v 1.7 2006/04/20 17:08:08 pborelli Exp $
+ * $Id: gedit-document-saver.c,v 1.10 2006/08/21 19:26:47 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,6 +50,12 @@
 #include "gedit-marshal.h"
 #include "gedit-utils.h"
 
+#ifdef HAVE_LIBATTR
+#include <attr/libattr.h>
+#else
+#define attr_copy_fd(x1, x2, x3, x4, x5, x6) (errno = ENOSYS, -1)
+#endif
+
 #define GEDIT_DOCUMENT_SAVER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), \
 						 GEDIT_TYPE_DOCUMENT_SAVER, \
 						 GeditDocumentSaverPrivate))
@@ -457,6 +463,16 @@ get_slow_mime_type (const char *text_uri)
 
 /* ----------- local files ----------- */
 
+#ifdef HAVE_LIBATTR
+/* Save everything: user/root xattrs, SELinux, ACLs. */
+static int
+all_xattrs (const char *xattr G_GNUC_UNUSED,
+	    struct error_context *err G_GNUC_UNUSED)
+{
+	return 1;
+}
+#endif
+
 static gboolean
 save_existing_local_file (GeditDocumentSaver *saver)
 {
@@ -597,6 +613,25 @@ save_existing_local_file (GeditDocumentSaver *saver)
 			goto fallback_strategy;
 		}
 
+		/* copy the xattrs, like user.mime_type, over. Also ACLs and
+		 * SELinux context. */
+		if ((attr_copy_fd (saver->priv->local_path,
+				   saver->priv->fd,
+				   tmp_filename,
+				   tmpfd,
+				   all_xattrs,
+				   NULL) == -1) &&
+		    (errno != EOPNOTSUPP) && (errno != ENOSYS))
+		{
+			gedit_debug_message (DEBUG_SAVER, "could not set xattrs");
+
+			close (tmpfd);
+			unlink (tmp_filename);
+			g_free (tmp_filename);
+
+			goto fallback_strategy;
+		}
+
 		if (!write_document_contents (tmpfd,
 					      GTK_TEXT_BUFFER (saver->priv->document),
 			 	 	      saver->priv->encoding,
@@ -672,8 +707,24 @@ save_existing_local_file (GeditDocumentSaver *saver)
 
 		saver->priv->mime_type = get_slow_mime_type (saver->priv->uri);
 
-		if (!saver->priv->keep_backup)
+		if (saver->priv->keep_backup)
+		{
+			/* remove executable permissions from the backup 
+			 * file */
+			mode_t new_mode = statbuf.st_mode;
+
+			new_mode &= ~(S_IXUSR | S_IXGRP | S_IXOTH);
+			
+			if (new_mode != statbuf.st_mode)
+			{
+				/* try to change permissions */
+				chmod (backup_filename, new_mode);
+			}
+		}
+		else
+		{
 			unlink (backup_filename);
+		}
 
 		close (tmpfd);
 
@@ -709,9 +760,11 @@ save_existing_local_file (GeditDocumentSaver *saver)
 			goto out;
 		}
 
+		/* open the backup file with the same permissions
+		 * except the executable ones */
 		bfd = open (backup_filename,
 			    O_WRONLY | O_CREAT | O_EXCL,
-			    statbuf.st_mode & 0777);
+			    statbuf.st_mode & 0666);
 
 		if (bfd == -1)
 		{
@@ -734,8 +787,8 @@ save_existing_local_file (GeditDocumentSaver *saver)
 			gedit_debug_message (DEBUG_SAVER, "could not restore group");
 
 			if (fchmod (bfd,
-			            (statbuf.st_mode& 0707) |
-			            ((statbuf.st_mode & 07) << 3)) != 0)
+			            (statbuf.st_mode& 0606) |
+			            ((statbuf.st_mode & 06) << 3)) != 0)
 			{
 				gedit_debug_message (DEBUG_SAVER, "could not even clear group perms");
 
@@ -751,6 +804,29 @@ save_existing_local_file (GeditDocumentSaver *saver)
 			}
 		}
 
+		/* copy the xattrs, like user.mime_type, over. Also ACLs and
+		 * SELinux context. */
+		if ((attr_copy_fd (saver->priv->local_path,
+				   saver->priv->fd,
+				   backup_filename,
+				   bfd,
+				   all_xattrs,
+				   NULL) == -1) &&
+		    (errno != EOPNOTSUPP) && (errno != ENOSYS))
+		{
+			gedit_debug_message (DEBUG_SAVER, "could not set xattrs");
+
+			g_set_error (&saver->priv->error,
+				     GEDIT_DOCUMENT_ERROR,
+				     GEDIT_DOCUMENT_ERROR_CANT_CREATE_BACKUP,
+				     "No backup created");
+
+			unlink (backup_filename);
+			close (bfd);
+
+			goto out;
+		}
+
 		if (!copy_file_data (saver->priv->fd, bfd, NULL))
 		{
 				gedit_debug_message (DEBUG_SAVER, "could not copy data into the backup");
@@ -776,6 +852,7 @@ save_existing_local_file (GeditDocumentSaver *saver)
 			 	      saver->priv->encoding,
 				      &saver->priv->error))
 	{
+		/* FIXME: restore the backup? */
 		goto out;
 	}
 
@@ -1189,7 +1266,7 @@ save_remote_file_real (GeditDocumentSaver *saver)
 
 	if (saver->priv->tmpfd == -1)
 	{
-		GnomeVFSResult result = gnome_vfs_result_from_errno ();
+		result = gnome_vfs_result_from_errno ();
 
 		g_set_error (&saver->priv->error,
 			     GEDIT_DOCUMENT_ERROR,
